

/*
 * Simple demo, should work with any driver board
 *
 * Connect STEP, DIR as indicated
 *
 * Copyright (C)2015-2017 Laurentiu Badea
 *
 * This file may be redistributed under the terms of the MIT license.
 * A copy of this license has been included with this distribution in the file LICENSE.
 */
#include <Arduino.h>

#include <AccelStepper.h>

#include <TimerOne.h>

#include <MD_Parola.h>
#include <MD_MAX72xx.h>
#include <SPI.h>

// Define the number of devices we have in the chain and the hardware interface
// NOTE: These pin numbers will probably not work with your hardware and may
// need to be adapted
#define HARDWARE_TYPE MD_MAX72XX::FC16_HW
#define MAX_DEVICES 4
#define CLK_PIN   50
#define DATA_PIN  51
#define CS_PIN    52

// Hardware SPI connection
//MD_Parola P = MD_Parola(HARDWARE_TYPE, CS_PIN, MAX_DEVICES);
// Arbitrary output pins
MD_Parola P = MD_Parola(HARDWARE_TYPE, DATA_PIN, CLK_PIN, CS_PIN, MAX_DEVICES);

char buffer1 [10];
char buffer2 [10];



//SCHRITTMOTOREN:
#define DirXPin A1       //RAMPS1.4:A1
#define StepXPin A0      //RAMPS1.4:A0
#define DirYPin A7       //RAMPS1.4:A7
#define StepYPin A6      //RAMPS1.4:A6
#define DirZPin 48       //RAMPS1.4:48
#define StepZPin 46      //RAMPS1.4:46
#define DirE1Pin 34        //RAMPS1.4 E1: D34
#define StepE1Pin 36       //RAMPS1.4 E1: D36

#define EnableDriverX 38  //CNCV3,4:8, RAMPS1.4: D38
#define EnableDriverY A2  //CNCV3,4:8, RAMPS1.4: A2
#define EnableDriverZ A8  //CNCV3,4:8, RAMPS1.4: A8
#define EnableDriverE1 30 //CNCV3,4:8, RAMPS1.4: 24

//ENDSTOPS:
#define EndstopXPin 3   //RAMPS1.4: 3 (Xmin)
#define EndstopYPin 14  //RAMPS1.4: 14 (Ymin)
#define EndstopZPin 18   //RAMPS1.4: 18 (Zmin)
#define EndstopE1Pin 19   //RAMPS1.4: 2 (X-Max)


#define BatLength 40     //Länge des Schläger [mm]
#define BallDia   20    // Durchmesser Ball
#define BallRad   BallDia/2    // Radius Ball

//JOYSTICKS:
#define AnalogInBat1x    A11     // RAMPS1.4: A9
#define AnalogInBat1y    A12     // RAMPS1.4: A5
#define AnalogInBat2x    A9     // RAMPS1.4: A9
#define AnalogInBat2y    A5     // RAMPS1.4: A5


//HINDERNISSE
#define Hinderniss1xFaktor    0.33     //Faktor der Länge vom Spielfeld
#define Hinderniss1yFaktor    0.66     
#define Hinderniss2xFaktor    0.33     //Faktor der Länge vom Spielfeld
#define Hinderniss2yFaktor    0.33 

#define Hinderniss3xFaktor    0.66     //Faktor der Länge vom Spielfeld
#define Hinderniss3yFaktor    0.66     
#define Hinderniss4xFaktor    0.66     //Faktor der Länge vom Spielfeld
#define Hinderniss4yFaktor    0.33 




AccelStepper Xaxis(1, StepXPin, DirXPin); //AccelStepper mystepper(1, pinStep, pinDirection);
AccelStepper Yaxis(1, StepYPin, DirYPin); 
AccelStepper Zaxis(1, StepZPin, DirZPin); //AccelStepper mystepper(1, pinStep, pinDirection);
AccelStepper E1axis(1, StepE1Pin, DirE1Pin); 

//mystepper.setMaxSpeed(stepsPerSecond);
//mystepper.setAcceleration(stepsPerSecondSquared);

//    Position Based Control::

//mystepper.currentPosition();        Read the motor's current absolution position.
//mystepper.distanceToGo();           Read the distance the motor is from its destination position. This can be used to check if the motor has reached its final position.
//mystepper.moveTo(targetPosition);   Move the motor to a new absolute position. This returns immediately. Actual movement is caused by the run() function.
//mystepper.move(distance);           Move the motor (either positive or negative) relative to its current position. This returns immediately. Actual movement is caused by the run() function

//mystepper.run();                    Update the motor. This must be called repetitively to make the motor move.

// Speed Based Control:

//mystepper.setSpeed(stepsPerSecond); Set the speed, in steps per second. This function returns immediately. Actual motion is caused by called runSpeed().
//mystepper.runSpeed();               Update the motor. This must be called repetitively to make the motor move


#define STEPS_PER_MM_X 40   //DRV8825, 20Tooth Pulley, GT2 Belt, 8 Mircosteps
#define STEPS_PER_MM_Y 40   //DRV8825, 20Tooth Pulley, GT2 Belt, 8 Mircosteps
#define STEPS_PER_MM_Z 40    //DRV8825, 20Tooth Pulley, GT2 Belt, 8 Mircosteps
#define STEPS_PER_MM_E1 40

#define X_DIR_FAKTOR 10
#define Z_DIR_MAX_FAKTOR 9
#define E1_DIR_MAX_FAKTOR 9
#define Y_DIR_MAX_FAKTOR 9

#define Y_FIELDSIZE 190     // in mm
#define Z_FIELDSIZE Y_FIELDSIZE     // in mm
#define E1_FIELDSIZE Y_FIELDSIZE     // in mm
#define X_FIELDSIZE 320     // in mm

// PLAYFIELD SIZE DEFINITION
#define MAX_X STEPS_PER_MM_X * X_FIELDSIZE   //Maximaler Y Weg für Puck
#define MAX_Y STEPS_PER_MM_Y * Y_FIELDSIZE   //Maximaler X Weg für Puck
#define MAX_Z STEPS_PER_MM_Z * Z_FIELDSIZE   //Maximaler X Weg für Puck
#define MAX_E1 STEPS_PER_MM_E1 * E1_FIELDSIZE   //Maximaler X Weg für Puck

#define MIN_X STEPS_PER_MM_X*5   //Minimaler Y Weg für Puck
#define MIN_Y STEPS_PER_MM_Y*5   //Minimaler X Weg für Puck
#define MIN_Z STEPS_PER_MM_Z*2   //Minimaler X Weg für Puck
#define MIN_E1 STEPS_PER_MM_E1*2   //Minimaler X Weg für Puck

#define START_MOVE_Z STEPS_PER_MM_Z*2   //Weg des Pucks vom Schläger1 nach vorne beim Spiel Starten
#define START_MOVE_E1 STEPS_PER_MM_Z*2   //Weg des Pucks vom Schläger2 nach vorne beim Spiel Starten

#define BREAK_ZONE_X 200          //Schlitten (X-Richtung) wird in dieser Zone in den Wendepunkten gebremst um Schläge zu dämpfen [steps]
#define BREAK_ZONE_FAKTOR_X 0.2   // Faktor zum Bremsen in der Break Zone für X-Schlitten

#define BREAK_ZONE_Y 80           //Schlitten (Y-Richtung) wird in dieser Zone in den Wendepunkten gebremst um Schläge zu dämpfen [steps]
#define BREAK_ZONE_FAKTOR_Y 0.5   // Faktor zum Bremsen in der Break Zone für Y-Schlitten

#define GEWINNPUNKTE 3


// SPEED DEFINITION
#define HOME_SPEED 80             //mm per sec
#define STANDARD_SPEED_MM_SEC_X 10  //mm per sec
#define STANDARD_SPEED_MM_SEC_Y 10  //mm per sec
#define STANDARD_SPEED_MM_SEC_Z 10  //mm per sec
#define STANDARD_SPEED_MM_SEC_E1 10  //mm per sec

#define MAX_SPEED_MM_SEC 120       //mm per sec              

long XStepPos = 0;
long YStepPos = 0;
long ZStepPos = 0;
long E1StepPos = 0;

long ReversePoint = 0;

long XStepDeltaToGo = 0;

long Bat1StepPos = 0;
long Bat2StepPos = 0;

int XDir = 0;
int YDir = 0;
int ZDir = 0;
int E1Dir = 0;

int PunkteStandSpieler1 = 0;
int PunkteStandSpieler2 = 0;


int int_AnalogInBat1x = 0;
int int_AnalogInBat1y = 0;
int int_AnalogInBat2x = 0;
int int_AnalogInBat2y = 0;


boolean OutOfBat1; 
boolean OutOfBat2;
boolean Spieler1Startet = true;
boolean Hinderniss1Treffer = false;
boolean Hinderniss2Treffer = false;
boolean Hinderniss3Treffer = false;
boolean Hinderniss4Treffer = false;

int Status = 0;   

//  0: Init
//  1: Homen
//  2: Spiel Starten: Ball an Startposition fahren, Mitte vom Schläger
//  3: Ball am Schläger, Spiel noch nicht gestartet
//  4: Spielläuft
//  5:  
//  6: Anzeige Punktestand 
//  7: Gewinner kühren (5 Punkte erreicht)
//  8: Neues Spiel starten

void DisplayAktuellesErgebnis(){
P.displayZoneText(1, "-", PA_LEFT, 0, 0, PA_PRINT, PA_NO_EFFECT);
P.displayZoneText(2, "-", PA_RIGHT, 0, 0, PA_PRINT, PA_NO_EFFECT);
P.displayZoneText(0, itoa(PunkteStandSpieler2,buffer2,10), PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
P.displayZoneText(3, itoa(PunkteStandSpieler1,buffer1,10), PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
P.displayAnimate();
}

void DisplayNeuesErgebnis(){
P.displayZoneText(1, " ", PA_LEFT, 0, 0, PA_PRINT, PA_NO_EFFECT);
P.displayZoneText(2, " ", PA_RIGHT, 0, 0, PA_PRINT, PA_NO_EFFECT);
for (int i = 0; i < 21; i++){
if (Spieler1Startet == true && i%2 > 0){         //Spieler 2 hat einen Punkt gemacht
     P.displayZoneText(0, itoa(PunkteStandSpieler2,buffer2,10), PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);       //Spieler 2
      }
     else{
     P.displayZoneText(0, " ", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
     }
if (Spieler1Startet == false && i%2 > 0){          //Spieler 1 hat einen Punkt gemacht
     P.displayZoneText(3, itoa(PunkteStandSpieler1,buffer1,10), PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
     }
     else{
     P.displayZoneText(3, " ", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
     }

     P.displayAnimate();
     delay(100);
}
DisplayAktuellesErgebnis();
}




void setup() {
    //P.begin();
    P.begin(4);
    P.setZone(0, 0, 0);     //Zwei Zonen in der DotMatrix definieren
    P.setZone(1, 1, 1);
    P.setZone(2, 2, 2);     //Zwei Zonen in der DotMatrix definieren
    P.setZone(3, 3, 3);


    Xaxis.setMaxSpeed(MAX_SPEED_MM_SEC*STEPS_PER_MM_X); //mystepper.setMaxSpeed(stepsPerSecond);
    Zaxis.setMaxSpeed(MAX_SPEED_MM_SEC*STEPS_PER_MM_Z);
    E1axis.setMaxSpeed(MAX_SPEED_MM_SEC*STEPS_PER_MM_E1);
    Yaxis.setMaxSpeed(MAX_SPEED_MM_SEC*STEPS_PER_MM_Y);
    Xaxis.setAcceleration(STEPS_PER_MM_X*80);   //Xaxis.setAcceleration(stepsPerSecondSquared);
    Yaxis.setAcceleration(STEPS_PER_MM_Y*80);
    Zaxis.setAcceleration(STEPS_PER_MM_Z*80);
    E1axis.setAcceleration(300);


    Serial.begin(9600);

    pinMode(EndstopXPin, INPUT);
    pinMode(EndstopYPin, INPUT);
    pinMode(EndstopZPin, INPUT);
    pinMode(EndstopE1Pin, INPUT);
    pinMode(AnalogInBat1x, INPUT);
    pinMode(AnalogInBat1y, INPUT);
    pinMode(AnalogInBat2x, INPUT);
    pinMode(AnalogInBat2y, INPUT);
    
    
    pinMode(EnableDriverZ, OUTPUT);
    pinMode(EnableDriverX, OUTPUT);
    pinMode(EnableDriverY, OUTPUT);
    pinMode(EnableDriverE1, OUTPUT);
    
    pinMode(EndstopXPin, INPUT_PULLUP);
    pinMode(EndstopYPin, INPUT_PULLUP);
    pinMode(EndstopZPin, INPUT_PULLUP);
    pinMode(EndstopE1Pin, INPUT_PULLUP);

    digitalWrite(EnableDriverE1,HIGH);
    digitalWrite(EnableDriverZ, HIGH);
    digitalWrite(EnableDriverX, HIGH);
    digitalWrite(EnableDriverY, HIGH);

    Timer1.initialize(10000); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
    Timer1.attachInterrupt( timerIsr ); // attach the service routine here
}

void loop() {

if (Status == 0){     //INIT, Start des Spiels über Joystick nach Oben drücken
  //Serial.println(int_AnalogInBat1x);  
  Serial.println(int_AnalogInBat2x); 
  Serial.println(E1Dir);
  delay(100);
  if (analogRead(AnalogInBat1y) < 100){
      Status = 1;
      digitalWrite(EnableDriverX, LOW);
      digitalWrite(EnableDriverY, LOW);
      digitalWrite(EnableDriverZ, LOW);
      digitalWrite(EnableDriverE1, LOW);
  }
}
  
if (Status == 1) {   //Homen

  P.displayZoneText(0, "E", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
  P.displayZoneText(1, "M", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
  P.displayZoneText(2, "O", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
  P.displayZoneText(3, "H", PA_CENTER, 0, 0, PA_PRINT, PA_NO_EFFECT);
  P.displayAnimate();
  Serial.println("Homen");
  Serial.println(EndstopXPin);
  Xaxis.setSpeed(-1*HOME_SPEED*STEPS_PER_MM_X); 
  Yaxis.setSpeed(-1*HOME_SPEED*STEPS_PER_MM_Y); 
  Zaxis.setSpeed(-1*HOME_SPEED*STEPS_PER_MM_Z);
  E1axis.setSpeed(1*HOME_SPEED*STEPS_PER_MM_E1);      //E1 homed nach +Richtung (Zmax)
  while (digitalRead(EndstopXPin) == HIGH){
    Xaxis.runSpeed();}
    
  Xaxis.setCurrentPosition(0);
  XStepPos = Xaxis.currentPosition();        //Read the motor's current absolution position.
  Serial.println(XStepPos);
  Serial.println("X-gehomed");
  
  while (digitalRead(EndstopYPin) == HIGH){
    Yaxis.runSpeed();}
  Serial.println("Y-gehomed");
  Yaxis.setCurrentPosition(0);
  YStepPos = Yaxis.currentPosition();        //Read the motor's current absolution position.
  
  while (digitalRead(EndstopZPin) == HIGH){  //Schläger 1 homen
    Zaxis.runSpeed();}
  Serial.println("Z-gehomed");
  Zaxis.setCurrentPosition(0);
  ZStepPos = Zaxis.currentPosition();        //Read the motor's current absolution position.

  while (digitalRead(EndstopE1Pin) == HIGH){  //Schläger 1 homen
  E1axis.runSpeed();}
  Serial.println("E1-gehomed");
  E1axis.setCurrentPosition(MAX_E1);
  E1StepPos = E1axis.currentPosition();        //Read the motor's current absolution position.
  
  Status = 2;
  P.displayClear();
    }


if (Status == 2){       // Ball auf Startpos am Schläger fahren, etwas nach vorne Fahren
  DisplayAktuellesErgebnis();
  Serial.println("Auf auf Start Fahren");
  digitalWrite(EnableDriverX, LOW);
  digitalWrite(EnableDriverY, LOW);
  digitalWrite(EnableDriverZ, LOW);
  digitalWrite(EnableDriverE1, LOW);
  Hinderniss1Treffer = false;
  Hinderniss2Treffer = false;
  Hinderniss3Treffer = false;
  Hinderniss4Treffer = false;
  
  

  if (Spieler1Startet == true){           //Abhängig davon, welches Spieler startet muss hier anders gefahren werden
  Xaxis.setSpeed(1*STANDARD_SPEED_MM_SEC_X*STEPS_PER_MM_X);
  while (XStepPos < MIN_X + START_MOVE_Z){
  Xaxis.runSpeed();}
  Yaxis.moveTo(ZStepPos); 
  while (abs(Yaxis.distanceToGo()) > 0){
  Yaxis.run();}
  Status = 3;
  ZDir = 0;
  }
  else {
  Xaxis.setSpeed(-1*STANDARD_SPEED_MM_SEC_X*STEPS_PER_MM_X);
  while (XStepPos > MAX_X - START_MOVE_E1){
  Xaxis.runSpeed();}
  Yaxis.moveTo(E1StepPos); 
  while (abs(Yaxis.distanceToGo()) > 0){
  Yaxis.run();}
  Status = 3;
  E1Dir = 0; 
  }}


if (Status == 3){       // Ball mit dem Schläger führen

  digitalWrite(EnableDriverX, LOW);
  digitalWrite(EnableDriverY, LOW);
  digitalWrite(EnableDriverZ, LOW);
  digitalWrite(EnableDriverE1, LOW);
  

  if (Spieler1Startet == true){           //Abhängig davon, welches Spieler startet muss hier anders gefahren werden
    while (int_AnalogInBat1y > 100){
    Zaxis.setSpeed(ZDir*STANDARD_SPEED_MM_SEC_Z*STEPS_PER_MM_Z);
    E1axis.setSpeed(E1Dir*STANDARD_SPEED_MM_SEC_E1*STEPS_PER_MM_E1);
    Yaxis.setSpeed(ZDir*STANDARD_SPEED_MM_SEC_Z*STEPS_PER_MM_Z);
    Zaxis.runSpeed();
    E1axis.runSpeed();
    Yaxis.runSpeed();
    }
    Serial.println("los");
    Status = 4;
    XDir = X_DIR_FAKTOR;              //Start Richtung, to do , zweiten Schläger einbauen
    YDir = ZDir;           //Start Richtung
    }
  else{
    while (int_AnalogInBat2y > 100){
    Zaxis.setSpeed(ZDir*STANDARD_SPEED_MM_SEC_Z*STEPS_PER_MM_Z);
    E1axis.setSpeed(E1Dir*STANDARD_SPEED_MM_SEC_E1*STEPS_PER_MM_E1);
    Yaxis.setSpeed(E1Dir*STANDARD_SPEED_MM_SEC_Z*STEPS_PER_MM_Z);
    Zaxis.runSpeed();
    E1axis.runSpeed();
    Yaxis.runSpeed();
    }
  Serial.println("los");
  Status = 4;
  XDir = -1*X_DIR_FAKTOR;              //Start Richtung, to do , zweiten Schläger einbauen
  YDir = E1Dir;           //Start Richtung
  }
  
}

 if (Status == 4){
  
    if (((ZStepPos + BatLength*STEPS_PER_MM_Y/2 + BallDia*STEPS_PER_MM_Y) > YStepPos) && ((ZStepPos - BatLength*STEPS_PER_MM_Y/2 - BallDia*STEPS_PER_MM_Y) < YStepPos)){OutOfBat1 = false;}     //Ist der Ball auf Höhe des Schlägers
    else {OutOfBat1 = true;}
    if (((E1StepPos + BatLength*STEPS_PER_MM_Y/2 + BallDia*STEPS_PER_MM_Y) > YStepPos) && ((E1StepPos - BatLength*STEPS_PER_MM_Y/2 - BallDia*STEPS_PER_MM_Y) < YStepPos)){OutOfBat2 = false;}     //Ist der Ball auf Höhe des Schlägers
    else {OutOfBat2 = true;}

     //OutOfBat2 = false;     // Nur ein Schläger
    
    
     if (XStepPos > MAX_X && XDir > 0 && !OutOfBat2){   //An Bat 2 abprallen
          //Serial.println("OutOfBat2:");
          //Serial.println(OutOfBat2);
          XDir = -1*X_DIR_FAKTOR;                            //negative Richtung
          YDir = E1Dir;
          Hinderniss1Treffer = false;
          Hinderniss2Treffer = false;
          Hinderniss3Treffer = false;
          Hinderniss4Treffer = false;
          }
     
     if (XStepPos > MAX_X && XDir > 0 && OutOfBat2){    //Bat 2 verfehlen
        XDir = 0; 
        YDir = 0;
        Status = 2;    
        PunkteStandSpieler1++;   //Punkt erzielt Spieler 1
        Spieler1Startet = false;
        DisplayNeuesErgebnis();
        if (PunkteStandSpieler1 == GEWINNPUNKTE){
            Status = 7;}      //Gewinner Kühren    
            }
   
     
        if (XStepPos < MIN_X && XDir < 0 && !OutOfBat1){    //An Bat 1 abprallen
            //Serial.println("OutOfBat1:");
            //Serial.println(OutOfBat1);
            XDir = X_DIR_FAKTOR;                                     //Positive Richtung
            YDir = ZDir; 
            Hinderniss1Treffer = false;
            Hinderniss2Treffer = false;
            Hinderniss3Treffer = false;
            Hinderniss4Treffer = false;
            
        }
        if (XStepPos < MIN_X && XDir < 0 && OutOfBat1){    //Bat 1 verfehlen

        XDir = 0; 
        YDir = 0;
        Status = 2;    //Punkt erzielt Spieler 2
        PunkteStandSpieler2++;
        Spieler1Startet = true;
        DisplayNeuesErgebnis();
        if (PunkteStandSpieler2 == GEWINNPUNKTE){
        Status = 7;}      //Gewinner Kühren 
     }
      
     if (YStepPos > MAX_Y && YDir > 0){                         //Links gegeen die Wand prallen
     YDir = -1*abs(YDir);                                       //Richtung Umkehren
     }

     if (YStepPos < MIN_Y && YDir < 0){                         //Rechts gegeen die Wand prallen
     YDir = abs(YDir);                                          //Richtung Umkehren
     }

     if (XStepPos > (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss1xFaktor - BallRad*STEPS_PER_MM_X) && XStepPos < (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss1xFaktor + BallRad*STEPS_PER_MM_X) && YStepPos > (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss1yFaktor - BallRad*STEPS_PER_MM_Y) && YStepPos < (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss1yFaktor + BallRad*STEPS_PER_MM_Y) && Hinderniss1Treffer == false){                         //Hinderniss vorne Links
     Hinderniss1Treffer = true;
     YDir = -1*YDir;                                       //Richtung Umkehren, Hinderniss 1 (vorne links)
     }

     if (XStepPos > (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss2xFaktor - BallRad*STEPS_PER_MM_X) && XStepPos < (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss2xFaktor + BallRad*STEPS_PER_MM_X) && YStepPos > (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss2yFaktor - BallRad*STEPS_PER_MM_Y) && YStepPos < (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss2yFaktor + BallRad*STEPS_PER_MM_Y) && Hinderniss2Treffer == false){                         //Hinderniss vorne rechts
     Hinderniss2Treffer = true;
     YDir = -1*YDir;        //Richtung Umkehren, Hinderniss 2 (vorne rechts)
     }

     if (XStepPos > (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss3xFaktor - BallRad*STEPS_PER_MM_X) && XStepPos < (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss3xFaktor + BallRad*STEPS_PER_MM_X) && YStepPos > (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss3yFaktor - BallRad*STEPS_PER_MM_Y) && YStepPos < (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss3yFaktor + BallRad*STEPS_PER_MM_Y) && Hinderniss3Treffer == false){                         //Hinderniss hinten Links
     Hinderniss3Treffer = true;
     YDir = -1*YDir;                                       //Richtung Umkehren, Hinderniss 3 (hinten links)
     }

     if (XStepPos > (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss4xFaktor - BallRad*STEPS_PER_MM_X) && XStepPos < (X_FIELDSIZE*STEPS_PER_MM_X*Hinderniss4xFaktor + BallRad*STEPS_PER_MM_X) && YStepPos > (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss4yFaktor - BallRad*STEPS_PER_MM_Y) && YStepPos < (Y_FIELDSIZE*STEPS_PER_MM_Y*Hinderniss4yFaktor + BallRad*STEPS_PER_MM_Y) && Hinderniss4Treffer == false){                         //Hinderniss hinten rechts
     Hinderniss4Treffer = true;
     YDir = -1*YDir;                                     //Richtung Umkehren, Hinderniss 4 (hinten rechts
     }

     
     if (Status == 4){
     if (XStepPos < (MIN_X + BREAK_ZONE_X) && XDir < 0){    //Am unteren Ende (4mm) auf 30% Bremsen und etweder abprallen oder verfehlen
     XDir = map(XStepPos, MIN_X + BREAK_ZONE_X, MIN_X , -1*X_DIR_FAKTOR, -1);      //Bremsen neu
     //XDir = -1*BREAK_ZONE_FAKTOR_X*X_DIR_FAKTOR;                       //Bremsen alt
     }
     if (XStepPos < (MIN_X + BREAK_ZONE_X) && XDir > 0){    //Am unteren Ende (4mm) auf 30% Bremsen und etweder abprallen oder verfehlen
     XDir = map(XStepPos, MIN_X, MIN_X + BREAK_ZONE_X, 1, X_DIR_FAKTOR);      //Beschleunigen
     //XDir = -1*BREAK_ZONE_FAKTOR_X*X_DIR_FAKTOR;                       //Bremsen alt
     XDir = max(1,XDir);
     }
     if (XStepPos > (MAX_X - BREAK_ZONE_X) && XDir > 0){       //Am oberen Ende (4mm) auf 30% bremsen
     XDir = map(XStepPos, MAX_X - BREAK_ZONE_X, MAX_X, X_DIR_FAKTOR, 1);      //Bremsen neu 
     //XDir = BREAK_ZONE_FAKTOR_X*X_DIR_FAKTOR;                               //Bremsen alt
     }
     if (XStepPos > (MAX_X - BREAK_ZONE_X) && XDir < 0){       //Am oberen Ende (4mm) auf 30% beschleunigen
     XDir = map(XStepPos, MAX_X, MAX_X - BREAK_ZONE_X, -1, -1*X_DIR_FAKTOR);      //Bremsen neu 
     //XDir = BREAK_ZONE_FAKTOR_X*X_DIR_FAKTOR;                               //Bremsen alt
     XDir = min(-1,XDir);
     }}
     
     
     Xaxis.setSpeed(XDir*STANDARD_SPEED_MM_SEC_X*STEPS_PER_MM_X);
     Yaxis.setSpeed(YDir*STANDARD_SPEED_MM_SEC_Y*STEPS_PER_MM_Y);
     Zaxis.setSpeed(ZDir*STANDARD_SPEED_MM_SEC_Z*STEPS_PER_MM_Z);
     E1axis.setSpeed(E1Dir*STANDARD_SPEED_MM_SEC_E1*STEPS_PER_MM_E1);
     Xaxis.runSpeed();
     Yaxis.runSpeed();
     Zaxis.runSpeed();
     E1axis.runSpeed();

}

if (Status == 7){       //Gewinner Kühren
  DisplayNeuesErgebnis();
  DisplayNeuesErgebnis();
  PunkteStandSpieler1 = 0;
  PunkteStandSpieler2 = 0;
  DisplayAktuellesErgebnis();
  delay(3000);
  Status = 2;           //Ball an Startposition fahren, Mitte vom Schläger
  Serial.println("Status 2");
  }  
}

void timerIsr()
{
if (Status != 4){
int_AnalogInBat1y = analogRead(AnalogInBat1y);  
int_AnalogInBat2y = analogRead(AnalogInBat2y);
}
XStepPos = Xaxis.currentPosition();
YStepPos = Yaxis.currentPosition();
ZStepPos = Zaxis.currentPosition();      //Bat1
E1StepPos = E1axis.currentPosition();

int_AnalogInBat1x = analogRead(AnalogInBat1x);
int_AnalogInBat2x = analogRead(AnalogInBat2x);
ZDir = map(int_AnalogInBat1x, 5, 1000, Z_DIR_MAX_FAKTOR, -1*Z_DIR_MAX_FAKTOR); 
if (int_AnalogInBat1x > 400 && int_AnalogInBat1x < 600){ZDir = 0;}
if ((ZStepPos > MAX_Z && ZDir > 0) || (ZStepPos < (MIN_Z + STEPS_PER_MM_Z*2) && ZDir < 0)){        //Prüfen ob Z im Range
    ZDir = 0;}
E1Dir = map(int_AnalogInBat2x, 5, 1000, -1*E1_DIR_MAX_FAKTOR, E1_DIR_MAX_FAKTOR); 
if (int_AnalogInBat2x > 400 && int_AnalogInBat2x < 600){E1Dir = 0;}
if ((E1StepPos > (MAX_E1 - STEPS_PER_MM_E1*2) && E1Dir > 0) || (E1StepPos < MIN_E1 && E1Dir < 0)){        //Prüfen ob E1 im Range
    E1Dir = 0;}
  
}
